function r = rdist2r(r0,k,rd,SWAP_DISTFCN)
%rdist2r Inverse lens radial distortion applied to a distorted image point radius.
%        r = rdist2r(r0,k,rd)
%        Parameters and distortion model are described in r2rdist.
%
%        rdist2r is the inverse function to r2rdist. 
%        Since the inverse cannot be found in a closed form, it is obtained by
%        a rapidly converging iterative algorithm, using the Banach's fixed point theorem.
%
%        See also r2rdist.

%       Author       : (c) T. Werner and T. Pajdla, {werner,pajdla}@cmp.felk.cvut.cz
%                      07/09/98
%                      Center for Machine Perception, Czech Technical University, Prague
%       Documentation: ftp://cmp.felk.cvut.cz/pub/cmp/articles/werner/tr97-138.ps.gz     
%       Language     : Matlab 5.1, (c) MathWorks                         
%       Last change  : $Id: rdist2r.m,v 1.1 1998/09/07 15:54:15 Matlab Exp $
%       Status       : Ready ($Source: /home.others/Matlab/Matlab/radial/RCS/rdist2r.m,v $)


% Swap functions R2RDIST and RDIST2R for debugging ?
if exist('SWAP_DISTFCN') == 0,
  SWAP_DISTFCN = 0;
end
if SWAP_DISTFCN,
  r = r2rdist(r0,k,rd,0);
  return
end


N  = size(rd,2);  % number of points
EPSILON = 10*eps*max(abs(rd)); % tollerance for the iterative algorithm to stop
DIF = 1e-6;       % finite diference for computing derivative of r2rdist
NEWTON = 1;


%%
 % Computing inverse to R2RDIST is equal to solving
 %   f(r) = 0
 % where
 %   f(r) = rd - R2RDIST(r0,k,r)
 %
 % This equation has single root, we find it by iteration method:
 %   1. r = initial estimate;
 %   2. repeat r = r - lam*f(r);
 % If lam = 1/f'(r), Newton methods is used,
 % if lam = -1, simple iteration is used.
 %
 % The loop stops in one of the following situations:
 %   1. If |f(r)| < epsilon  ---> success
 %   2. If |f(r)| > huge     ---> failure due to divergence
 %%

% Find initial guess of r. See that 2*x-g(x) is a good 
% approximation of inverse of g(x) as long as g(x) is near x. 
r = rd; %2*rd - r2rdist(r0,k,rd);

% Iterations.
ir = 1:length(rd);
for n = 1:200
  f(ir) = rd(ir)-r2rdist(r0,k,r(ir));
  ir = abs(f) > EPSILON;
  if ~nnz(ir), break, end
  if max(abs(r)) > 100*abs(rd), disp('!!! Iteration diverges in rdist2r.'); break, end
  if NEWTON
    lam = 1./(rd(ir)-r2rdist(r0,k,r(ir)+DIF)-f(ir))*DIF;
  else
    lam = -1;
  end
  r(ir) = r(ir) - lam.*f(ir);
end
if n>20, disp(['Warning: ' num2str(n) ' iterations in rdist2r.']), end

%max(abs( rd - r2rdist(r0,k,r) )) 

return
